--=======================================================================================================
--  ROCK DISPOSAL SCRIPT
-- 
-- Author:		akuenzi (akuenzi@gmail.com)
-- Date:		12/16/2018
-- Purpose:		Adds placeable rock disposal point.
--=======================================================================================================


RockDisposalPoint = {};
RockDisposalPoint.ModDir = g_currentModDirectory

local RockDisposalPoint_mt = Class(RockDisposalPoint, Placeable);

InitObjectClass(RockDisposalPoint, "RockDisposalPoint");
registerPlaceableType("rockDisposalPoint", RockDisposalPoint);

function RockDisposalPoint:new(isServer, isClient, customMt)
    local mt = customMt;
    if mt == nil then
        mt = RockDisposalPoint_mt;
    end;

    local self = Placeable:new(isServer, isClient, mt);

    registerObjectClassName(self, "RockDisposalPoint");
		
    return self;
end;

function RockDisposalPoint:delete()
    if self.tipTrigger ~= nil then
        self.tipTrigger:removeUpdateEventListener(self)
        if self.tipTrigger.isRegistered then
            self.tipTrigger:unregister()
        end;
        self.tipTrigger:delete()
    end;
	
	if self.mapHotspot ~= nil then
		g_currentMission.ingameMap:deleteMapHotspot(self.mapHotspot)
	end;

    unregisterObjectClassName(self);
    RockDisposalPoint:superClass().delete(self);
end;

function RockDisposalPoint:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
    if not RockDisposalPoint:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
        return false;
    end;

    local xmlFile = loadXMLFile("TempXML", xmlFilename);

    self.tipTrigger = TipTrigger:new(self.isServer, self.isClient)
    if self.tipTrigger:load(Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.rockDisposalPoint#tipTrigger"))) then
        self.tipTrigger:register(true)
        self.tipTrigger:addUpdateEventListener(self)
    else
        self.tipTrigger:delete()
        self.tipTrigger = nil
    end;

	-- To create map hotspot for the rock disposal point.
	local name = 			"RockDisposal"
	local fullName =		g_i18n:getText("ROCK_DISPOSAL_HOTSPOT")
	local file = 			"Rock_Disposal_Point/iconHotspot.dds"
	local imageFilename =	Utils.getFilename(file, RockDisposalPoint.ModDir)
	local objectId =		self.nodeId
	
	self.mapHotspot = g_currentMission.ingameMap:createMapHotspot(name, fullName, imageFilename, nil, nil, x, z, nil, nil, false, false, true, objectId, nil, MapHotspot.CATEGORY_DEFAULT)
	--function IngameMap:createMapHotspot(name, fullName, imageFilename, imageUVs, baseColor, xMapPos, zMapPos, width, height, blinking, persistent, showName, objectId, renderLast, category, textSize, textOffsetY, textColor, bgImageFilename, bgImageUVs, verticalAlignment, overlayBgScale)
	
    delete(xmlFile);

    return true;
end;

function RockDisposalPoint:readStream(streamId, connection)
    RockDisposalPoint:superClass().readStream(self, streamId, connection);
    if connection:getIsServer() then
        local tipTriggerId = readNetworkNodeObjectId(streamId);
        self.tipTrigger:readStream(streamId, connection);
        g_client:finishRegisterObject(self.tipTrigger, tipTriggerId);
    end;
end;

function RockDisposalPoint:writeStream(streamId, connection)
    RockDisposalPoint:superClass().writeStream(self, streamId, connection);
    if not connection:getIsServer() then
        writeNetworkNodeObjectId(streamId, self.tipTrigger.id);
        self.tipTrigger:writeStream(streamId, connection);
        g_server:registerObjectInStream(connection, self.tipTrigger);
    end;
end;

function RockDisposalPoint:collectPickObjects(node)
    if self.tipTrigger == nil or self.tipTrigger.shovelTarget == nil or self.tipTrigger.shovelTarget.nodeId ~= node then
        RockDisposalPoint:superClass().collectPickObjects(self, node)
    end;
end;

function RockDisposalPoint:update(dt)
    RockDisposalPoint:superClass().update(self, dt);
end;

function RockDisposalPoint:updateTick(dt)
    RockDisposalPoint:superClass().updateTick(self, dt);
end;

function RockDisposalPoint:onUpdateEvent(trigger, fillDelta, fillType, trailer, tipTriggerTarget)
end